###
Part of codes borrow from mongoose
https://github.com/LearnBoost/mongoose/blob/master/lib/mongoose/connection.js

@param path e.g. 'db://localhost:27017/blog?auto_reconnect=true'
@param options the options of Db
@param callback function(error, db)
@return db
###
url = require 'url'
{Db, Server, ServerCluster, BSONNative} = require 'mongodb'

STATUS_CLOSE = 0
STATUS_OPENNING = 1
STATUS_OPEN = 2

_extend = (destination, source) ->
  for property, value of source
    destination[property] = value
  return destination

class SkinServer

  constructor: (@nativeServer) ->
    @_cache_ = []

  db: (name, username, password) ->
    @_cache_[name] or= new SkinDb new Db(name, @nativeServer, {native_parser: BSONNative != undefined})


class SkinDb

  constructor: (@nativeDb, @username, @password) ->
    @status = 0

  open: (fn) ->
    switch @status
      when STATUS_OPEN
        fn null, @nativeDb
      when STATUS_OPENNING
        # call fn after open
        @once 'open', fn
      when STATUS_CLOSE
        that = this
        # call fn after open
        @once 'open', fn
        # after retrieval db or failed emit 'open'
        callback = (err, nativeDb) ->
          if not err and nativeDb
            that.status = STATUS_OPEN
            that.nativeDb = nativeDb
          that.emit 'open', err, nativeDb

        # start open
        @status = STATUS_OPENNING
        @nativeDb.open (err, nativeDb)->
          if err
            callback err
          #authenticate if need
          else if that.username
            nativeDb.authenticate that.username, that.password, (err) ->
              callback err, nativeDb
          else
            callback err, nativeDb

  # return a SkinCollection instance
  collection: (name) ->
    return @_collections[name] or= new SkinCollection this, name

  ###
     1. collection
     2. collection, extends1, extends2,... extendsn
     3. collection, SkinCollection 
  ###
  bind: (name, args...) ->
    if typeof name isnt 'string' or name.length is 0
      throw new Error('Must provide name parameter for bind.')
    if(args.length is 0)
      return @bind name, @collection(name)
    if(args.length is 1 and args[0].constructor is SkinCollection)
      @_collections[name] = args[0]
      Object.defineProperty(this, name, {
        value: args[0]
        writable: false
        enumerable: true
      })
      return args[0]
    flag=true
    flag and= typeof arg is 'object' for arg in args
    if not flag 
      throw new Error 'the arguments of bind should like (name, {...},{...})'
    coll = @collection arg
    coll = _extend coll, arg for arg in args
    @bind name, coll

bind = (name, method) ->
  SkinDb.prototype[name] = (args...) ->
    @open (err, nativeDb) ->
      if err
        args[args.length-1] err
      else
        method.apply nativeDb, args

bind name, method for name, method of Db.prototype when name not in ['bind', 'open', 'collection']


class MongoSkin
  
  constructor: ()->
    @__bindArgs__ = []

  bind: (args...) ->
    @__bindArgs__.push args

  pair: (left, right) ->
    return new SkinServer new ServerPair(new Server(left[0], left[1]||27017), new Server(right[0], right[1]||27017))

  cluster: (servers, auto_reconnect) ->
    serverList = []
    for config in servers
      serverList.push new Server(config[0], config[1] || 27017, auto_reconnect)
    return new SkinServer new ServerCluster serverList


  db: (serverUrl) ->
 
    uri = url.parse serverUrl, true
    host = uri.hostname
    port = uri.port || 27017
    database = uri.pathname.replace(/\//g, '')
    serverOptions = uri.query
    serverOptions['auto_reconnect'] = serverOptions['auto_reconnect'] is 'true'

    # handle authentication
    if uri and uri.auth
      auth = uri.auth.split(':')
      @username = auth[0]
      @password = auth[1]
    else 
      @username = @password = undefined
    
    if (!host) 
      throw new Error('Please provide a valid hostname.')

    if (!database) 
      throw new Error('Please provide a database to connect to.')

    server = new Server(host, port, serverOptions)
    nativeDb = new Db(database, server, {native_parser: BSONNative != undefined})
    db = new SkinDb nativeDb, username, password

    for bindArg in @__binds__
      db.bind bindArg...
    return db
